@@ -1,3 +1,4 @@
+#![allow(clippy::test_attr_in_doctest)]
 //! This crate will help you to write simpler tests by leveraging a software testing concept called
 //! [test fixtures](https://en.wikipedia.org/wiki/Test_fixture#Software). A fixture is something
 //! that you can use in your tests to encapsulate a test's dependencies.
@@ -92,6 +93,10 @@
 //!     assert!(string_processor.output.contains("Alice"));
 //! }
 //! ```
+//! ### Features
+//!
+//! - `async-timeout`: `timeout` for `async` tests (Default enabled)
+//! - `crate-name`: Import `rstest` package with different name (Default enabled)
 //!
 //! ## Injecting fixtures as function arguments
 //!
@@ -226,29 +231,24 @@
 //! }
 //! ```
 //! You can use this feature also in value list and in fixture default value.
+//!
+//! # Optional features
+//!
+//! `rstest` Enable all features by default. You can disable them if you need to
+//! speed up compilation.
+//!
+//! - **`async-timeout`** *(enabled by default)* — Implement timeout for async
+//! tests.
+//!
+//! # Rust version compatibility
+//!
+//! The minimum supported Rust version is 1.67.1.
+//!
 
-#![cfg_attr(use_proc_macro_diagnostic, feature(proc_macro_diagnostic))]
-extern crate proc_macro;
-
-// Test utility module
-#[cfg(test)]
-pub(crate) mod test;
-#[cfg(test)]
-use rstest_reuse;
-
-#[macro_use]
-mod error;
-mod parse;
-mod refident;
-mod render;
-mod resolver;
-mod utils;
-
-use syn::{parse_macro_input, ItemFn};
-
-use crate::parse::{fixture::FixtureInfo, future::ReplaceFutureAttribute, rstest::RsTestInfo};
-use parse::ExtendWithFunctionAttrs;
-use quote::ToTokens;
+#[doc(hidden)]
+pub mod magic_conversion;
+#[doc(hidden)]
+pub mod timeout;
 
 /// Define a fixture that you can use in all `rstest`'s test arguments. You should just mark your
 /// function as `#[fixture]` and then use it as a test's argument. Fixture functions can also
@@ -366,12 +366,29 @@ use quote::ToTokens;
 /// }
 /// ```
 ///
+/// This feature can also be useful when you don't want to declare the `use` of a fixture or simple
+/// use the fixture's path:
+///
+/// ```
+/// use rstest::*;
+///
+/// # mod magic_numbers {
+/// #     use rstest::*;
+/// #     #[fixture]
+/// #     pub fn fortytwo() -> i32 { 42 }
+/// # }
+/// #[rstest]
+/// fn the_test(#[from(magic_numbers::fortytwo)] x: i32) {
+///     assert_eq!(42, x)
+/// }
+/// ```
+///
 /// # `#[once]` Fixture
 ///
-/// Expecially in integration tests there are cases where you need a fixture that is called just once
+/// Especially in integration tests there are cases where you need a fixture that is called just once
 /// for every tests. `rstest` provides `#[once]` attribute for these cases.
 ///
-/// If you mark your fixture with this attribute and `rstest` will compute a static reference to your
+/// If you mark your fixture with this attribute, then `rstest` will compute a static reference to your
 /// fixture result and return this reference to all your tests that need this fixture.
 ///
 /// In follow example all tests share the same reference to the `42` static value.
@@ -383,13 +400,13 @@ use quote::ToTokens;
 /// #[once]
 /// fn once_fixture() -> i32 { 42 }
 ///
-/// // Take care!!! You need tu use a reference to fixture value
+/// // Take care!!! You need to use a reference to the fixture value
 ///
 /// #[rstest]
 /// #[case(1)]
 /// #[case(2)]
 /// fn cases_tests(once_fixture: &i32, #[case] v: i32) {
-///     // Take care!!! You need tu use a reference to fixture value
+///     // Take care!!! You need to use a reference to the fixture value
 ///     assert_eq!(&42, once_fixture)
 /// }
 ///
@@ -405,11 +422,11 @@ use quote::ToTokens;
 /// - `async` function
 /// - Generic function (both with generic types or use `impl` trait)
 ///
-/// Take care that the `#[once]` fixture value will **never dropped**.
+/// Take care that the `#[once]` fixture value will **never be dropped**.
 ///
 /// # Partial Injection
 ///
-/// You can also partialy inject fixture dependency using `#[with(v1, v2, ..)]` attribute:
+/// You can also partially inject fixture dependency using `#[with(v1, v2, ..)]` attribute:
 ///
 /// ```
 /// use rstest::*;
@@ -435,7 +452,7 @@ use quote::ToTokens;
 /// attribute will inject `v1, ..., vn` expression as fixture arguments: all remaining arguments
 /// will be resolved as fixtures.
 ///
-/// Sometimes the return type cannot be infered so you must define it: For the few times you may
+/// Sometimes the return type cannot be inferred so you must define it: For the few times you may
 /// need to do it, you can use the `#[default(type)]`, `#[partial_n(type)]` function attribute
 /// to define it:
 ///
@@ -474,8 +491,8 @@ use quote::ToTokens;
 ///
 /// # Old _compact_ syntax
 ///
-/// There is also a compact form for all previous features. This will mantained for a long time
-/// but for `fixture` I strongly recomand to migrate your code because you'll pay a little
+/// There is also a compact form for all previous features. This will maintained for a long time
+/// but for `fixture` I strongly recommend to migrate your code because you'll pay a little
 /// verbosity but get back a more readable code.
 ///
 /// Follow the previous examples in old _compact_ syntax.
@@ -534,34 +551,7 @@ use quote::ToTokens;
 ///     std::iter::once((i, j))
 /// }
 /// ```
-
-#[proc_macro_attribute]
-pub fn fixture(
-    args: proc_macro::TokenStream,
-    input: proc_macro::TokenStream,
-) -> proc_macro::TokenStream {
-    let mut info: FixtureInfo = parse_macro_input!(args as FixtureInfo);
-    let mut fixture = parse_macro_input!(input as ItemFn);
-
-    let replace_result = ReplaceFutureAttribute::replace(&mut fixture);
-    let extend_result = info.extend_with_function_attrs(&mut fixture);
-
-    let mut errors = error::fixture(&fixture, &info);
-
-    if let Err(attrs_errors) = replace_result {
-        attrs_errors.to_tokens(&mut errors);
-    }
-    if let Err(attrs_errors) = extend_result {
-        attrs_errors.to_tokens(&mut errors);
-    }
-
-    if errors.is_empty() {
-        render::fixture(fixture, info)
-    } else {
-        errors
-    }
-    .into()
-}
+pub use rstest_macros::fixture;
 
 /// The attribute that you should use for your tests. Your
 /// annotated function's arguments can be
@@ -570,7 +560,7 @@ pub fn fixture(
 /// [parametrized cases](attr.rstest.html#test-parametrized-cases)
 /// or by [value lists](attr.rstest.html#values-lists).
 ///
-/// `rstest` attribute can be applied to _any_ function and you can costumize its
+/// `rstest` attribute can be applied to _any_ function and you can customize its
 /// parameters by using function and arguments attributes.
 ///
 /// Your test function can use generics, `impl` or `dyn` and like any kind of rust tests:
@@ -593,7 +583,7 @@ pub fn fixture(
 /// The simplest case is write a test that can be injected with
 /// [`[fixture]`](macro@fixture)s. You can just declare all used fixtures by passing
 /// them as a function's arguments. This can help your test to be neat
-/// and make your dependecy clear.
+/// and make your dependency clear.
 ///
 /// ```
 /// use rstest::*;
@@ -634,7 +624,7 @@ pub fn fixture(
 /// }
 /// ```
 ///
-/// Sometimes is useful to have some parametes in your fixtures but your test would
+/// Sometimes is useful to have some parameters in your fixtures but your test would
 /// override the fixture's default values in some cases. Like in
 /// [fixture partial injection](attr.fixture.html#partial-injection) you use `#[with]`
 /// attribute to indicate some fixture's arguments also in `rstest`.
@@ -685,8 +675,8 @@ pub fn fixture(
 /// }
 /// ```
 ///
-/// `rstest` will produce 5 indipendent tests and not just one that
-/// check every case. Every test can fail indipendently and `cargo test`
+/// `rstest` will produce 5 independent tests and not just one that
+/// check every case. Every test can fail independently and `cargo test`
 /// will give follow output:
 ///
 /// ```text
@@ -700,7 +690,7 @@ pub fn fixture(
 /// test result: ok. 5 passed; 0 failed; 0 ignored; 0 measured; 0 filtered out
 /// ```
 ///
-/// The cases input values can be arbitrary Rust expresions that return the
+/// The cases input values can be arbitrary Rust expressions that return the
 /// argument type.
 ///
 /// ```
@@ -716,6 +706,24 @@ pub fn fixture(
 ///     assert_eq!(s.as_ref().len(), len);
 /// }
 /// ```
+/// ### Feature flagged cases
+///
+/// In case you want certain test cases to only be present if a certain feature is
+/// enabled, use `#[cfg_attr(feature = …, case(…))]`:
+///
+/// ```
+/// use rstest::rstest;
+///
+/// #[rstest]
+/// #[case(2, 2)]
+/// #[cfg_attr(feature = "frac", case(4/2, 2))]
+/// #[case(4/2, 2)]
+/// fn it_works(#[case] a: u32, #[case] b: u32) {
+///     assert!(a == b);
+/// }
+/// ```
+///
+/// This also works with [`rstest_reuse`](https://crates.io/crates/rstest_reuse).
 ///
 /// ### Magic Conversion
 ///
@@ -728,7 +736,7 @@ pub fn fixture(
 /// # fn count_words(path: PathBuf) -> usize {0}
 /// #[rstest]
 /// #[case("resources/empty", 0)]
-/// #[case("resources/divine_commedy", 101.698)]
+/// #[case("resources/divine_comedy", 101.698)]
 /// fn test_count_words(#[case] path: PathBuf, #[case] expected: usize) {
 ///     assert_eq!(expected, count_words(path))
 /// }
@@ -760,7 +768,7 @@ pub fn fixture(
 /// # }
 /// ```
 ///
-/// Outuput will be
+/// Output will be
 /// ```text
 /// running 4 tests
 /// test fibonacci_test::case_1_zero_base_case ... ok
@@ -829,7 +837,7 @@ pub fn fixture(
 /// ## Values Lists
 ///
 /// Another useful way to write a test and execute it for some values
-/// is to use the values list syntax. This syntax can be usefull both
+/// is to use the values list syntax. This syntax can be useful both
 /// for a plain list and for testing all combination of input arguments.
 ///
 /// ```
@@ -838,7 +846,7 @@ pub fn fixture(
 ///
 /// #[rstest]
 /// fn should_be_valid(
-///     #[values("Jhon", "alice", "My_Name", "Zigy_2001")]
+///     #[values("John", "alice", "My_Name", "Zigy_2001")]
 ///     input: &str
 /// ) {
 ///     assert!(is_valid(input))
@@ -864,35 +872,65 @@ pub fn fixture(
 /// where `cargo test` output is
 ///
 /// ```text
-/// running 6 tests
-/// test should_accept_all_corner_cases::name_1::age_1 ... ok
-/// test should_accept_all_corner_cases::name_3::age_1 ... ok
-/// test should_accept_all_corner_cases::name_3::age_2 ... ok
-/// test should_accept_all_corner_cases::name_2::age_1 ... ok
-/// test should_accept_all_corner_cases::name_2::age_2 ... ok
-/// test should_accept_all_corner_cases::name_1::age_2 ... ok
+/// test should_accept_all_corner_cases::name_1___J__::age_2_100 ... ok
+/// test should_accept_all_corner_cases::name_2___A__::age_1_14 ... ok
+/// test should_accept_all_corner_cases::name_2___A__::age_2_100 ... ok
+/// test should_accept_all_corner_cases::name_3___A________________________________________21__::age_2_100 ... ok
+/// test should_accept_all_corner_cases::name_3___A________________________________________21__::age_1_14 ... ok
+/// test should_accept_all_corner_cases::name_1___J__::age_1_14 ... ok
 ///
-/// test result: ok. 6 passed; 0 failed; 0 ignored; 0 measured; 0 filtered out
+/// test result: ok. 6 passed; 0 failed; 0 ignored; 0 measured; 0 filtered out; finished in 0.00s
 /// ```
+/// Note that the test names contains the given expression sanitized into
+/// a valid Rust identifier name. This should help to identify which case fails.
+///
 ///
 /// Also value list implements the magic conversion feature: every time the value type
 /// implements `FromStr` trait you can use a literal string to define it.
 ///
-/// ## Use Parametrize definition in more tests
-///
-/// If you need to use a test list for more than one test you can use
-/// [`rstest_reuse`](https://crates.io/crates/rstest_reuse) crate.
-/// With this helper crate you can define a template and use it everywhere.
-///
 /// ```
 /// # use rstest::rstest;
 /// # use std::net::SocketAddr;
 /// #[rstest]
 /// fn given_port(#[values("1.2.3.4:8000", "4.3.2.1:8000", "127.0.0.1:8000")] addr: SocketAddr) {
-///     assert_eq(8000, addr.port())
+///     assert_eq!(8000, addr.port())
 /// }
 /// ```
 ///
+/// ## Files path as input arguments
+///
+/// If you need to create a test for each file in a given location you can use
+/// `#[files("glob path syntax")]` attribute to generate a test for each file that
+/// satisfy the given glob path.
+///
+/// ```
+/// # use rstest::rstest;
+/// # use std::path::{Path, PathBuf};
+/// # fn check_file(path: &Path) -> bool { true };
+/// #[rstest]
+/// fn for_each_file(#[files("src/**/*.rs")] #[exclude("test")] path: PathBuf) {
+///     assert!(check_file(&path))
+/// }
+/// ```
+/// The default behavior is to ignore the files that start with `"."`, but you can
+/// modify this by use `#[include_dot_files]` attribute. The `files` attribute can be
+/// used more than once on the same variable, and you can also create some custom
+/// exclusion rules with the `#[exclude("regex")]` attributes that filter out all
+/// paths that verify the regular expression.
+///
+/// Sometime is useful to have test files in a workspace folder to share them between the
+/// crates in your workspace. You can do that by use the usual parent folders `..` in
+/// the glob path. In this case the test names will be the relative path from the crate root
+/// where the parent folder components are replaced by `_UP`: for instance if you have a
+/// `valid_call.yaml` in the folder `../test_cases` (from your crate root) a test name could be
+/// `path_1__UP_test_cases_valid_call_yaml`.
+///
+/// ## Use Parametrize definition in more tests
+///
+/// If you need to use a test list for more than one test you can use
+/// [`rstest_reuse`](https://crates.io/crates/rstest_reuse) crate.
+/// With this helper crate you can define a template and use it everywhere.
+///
 /// ```rust,ignore
 /// use rstest::rstest;
 /// use rstest_reuse::{self, *};
@@ -905,11 +943,11 @@ pub fn fixture(
 ///
 /// #[apply(two_simple_cases)]
 /// fn it_works(#[case] a: u32,#[case] b: u32) {
-///     assert!(a == b);
+///     assert_eq!(a, b);
 /// }
 /// ```
 ///
-/// See [`rstest_reuse`](https://crates.io/crates/rstest_reuse) for more dettails.
+/// See [`rstest_reuse`](https://crates.io/crates/rstest_reuse) for more details.
 ///
 /// ## Async
 ///
@@ -959,6 +997,93 @@ pub fn fixture(
 /// }
 /// ```
 ///
+/// As you noted you should `.await` all _future_ values and this some times can be really boring.
+/// In this case you can use `#[future(awt)]` to _awaiting_ an input or annotating your function
+/// with `#[awt]` attributes to globally `.await` all your _future_ inputs. Previous code can be
+/// simplified like follow:
+///
+/// ```
+/// use rstest::*;
+/// # #[fixture]
+/// # async fn base() -> u32 { 42 }
+///
+/// #[rstest]
+/// #[case(21, async { 2 })]
+/// #[case(6, async { 7 })]
+/// #[awt]
+/// async fn global(#[future] base: u32, #[case] expected: u32, #[future] #[case] div: u32) {
+///     assert_eq!(expected, base / div);
+/// }
+///
+/// #[rstest]
+/// #[case(21, async { 2 })]
+/// #[case(6, async { 7 })]
+/// async fn single(#[future] base: u32, #[case] expected: u32, #[future(awt)] #[case] div: u32) {
+///     assert_eq!(expected, base.await / div);
+/// }
+/// ```
+///
+/// ### Default timeout
+///
+/// You can set a default timeout for test using the `RSTEST_TIMEOUT` environment variable.
+/// The value is in seconds and is evaluated on test compile time.///
+///
+/// ### Test `#[timeout()]`
+///
+/// You can define an execution timeout for your tests with `#[timeout(<duration>)]` attribute. Timeout
+/// works both for sync and async tests and is runtime agnostic. `#[timeout(<duration>)]` take an
+/// expression that should return a `std::time::Duration`. Follow a simple async example:
+///
+/// ```rust
+/// use rstest::*;
+/// use std::time::Duration;
+///
+/// async fn delayed_sum(a: u32, b: u32,delay: Duration) -> u32 {
+///     async_std::task::sleep(delay).await;
+///     a + b
+/// }
+///
+/// #[rstest]
+/// #[timeout(Duration::from_millis(80))]
+/// async fn single_pass() {
+///     assert_eq!(4, delayed_sum(2, 2, ms(10)).await);
+/// }
+/// ```
+/// In this case test pass because the delay is just 10 milliseconds and timeout is
+/// 80 milliseconds.
+///
+/// You can use `timeout` attribute like any other attribute in your tests, and you can
+/// override a group timeout with a test specific one. In the follow example we have
+/// 3 tests where first and third use 100 milliseconds but the second one use 10 milliseconds.
+/// Another valuable point in this example is to use an expression to compute the
+/// duration.
+///
+/// ```rust
+/// # use rstest::*;
+/// # use std::time::Duration;
+/// #
+/// # async fn delayed_sum(a: u32, b: u32,delay: Duration) -> u32 {
+/// #     async_std::task::sleep(delay).await;
+/// #     a + b
+/// # }
+/// fn ms(ms: u32) -> Duration {
+///     Duration::from_millis(ms.into())
+/// }
+///
+/// #[rstest]
+/// #[case::pass(ms(1), 4)]
+/// #[timeout(ms(10))]
+/// #[case::fail_timeout(ms(60), 4)]
+/// #[case::fail_value(ms(1), 5)]
+/// #[timeout(ms(100))]
+/// async fn group_one_timeout_override(#[case] delay: Duration, #[case] expected: u32) {
+///     assert_eq!(expected, delayed_sum(2, 2, delay).await);
+/// }
+/// ```
+///
+/// If you want to use `timeout` for `async` test you need to use `async-timeout`
+/// feature (enabled by default).
+///
 /// ## Inject Test Attribute
 ///
 /// If you would like to use another `test` attribute for your test you can simply
@@ -982,6 +1107,39 @@ pub fn fixture(
 /// in this case the `#[actix_rt::test]` attribute will replace the standard `#[test]`
 /// attribute.
 ///
+/// ## Local lifetime and `#[by_ref]` attribute
+///
+/// In some cases you may want to use a local lifetime for some arguments of your test.
+/// In these cases you can use the `#[by_ref]` attribute then use the reference instead
+/// the value.
+///
+/// ```rust
+/// # use std::cell::Cell;
+/// # use rstest::*;
+/// # #[derive(Debug, Clone, Copy, PartialEq, Eq)]
+/// enum E<'a> {
+///     A(bool),
+///     B(&'a Cell<E<'a>>),
+/// }
+///
+/// fn make_e_from_bool<'a>(_bump: &'a (), b: bool) -> E<'a> {
+///     E::A(b)
+/// }
+///
+/// #[fixture]
+/// fn bump() -> () {}
+///  
+/// #[rstest]
+/// #[case(true, E::A(true))]
+/// fn it_works<'a>(#[by_ref] bump: &'a (), #[case] b: bool, #[case] expected: E<'a>) {
+///     let actual = make_e_from_bool(&bump, b);
+///     assert_eq!(actual, expected);
+/// }
+/// ```
+///
+/// You can use `#[by_ref]` attribute for all arguments of your test and not just for fixture
+/// but also for cases, values and files.
+///
 /// ## Putting all Together
 ///
 /// All these features can be used together with a mixture of fixture variables,
@@ -1011,13 +1169,13 @@ pub fn fixture(
 /// }
 ///
 /// #[rstest]
-/// #[case::authed_user(alice())] // We can use `fixture` also as standard function
+/// #[case::authorized_user(alice())] // We can use `fixture` also as standard function
 /// #[case::guest(User::Guest)]   // We can give a name to every case : `guest` in this case
-/// #[should_panic(expected = "Invalid query error")] // We whould test a panic
+/// #[should_panic(expected = "Invalid query error")] // We would test a panic
 /// fn should_be_invalid_query_error(
 ///     repository: impl Repository,
 ///     #[case] user: User,
-///     #[values("     ", "^%$#@!", "....")]
+///     #[values("     ", "^%$some#@invalid!chars", ".n.o.d.o.t.s.")] query: &str,
 ///     query: &str
 /// ) {
 ///     repository.find_items(&user, query).unwrap();
@@ -1086,7 +1244,7 @@ pub fn fixture(
 ///
 /// `rstest` support also a syntax where all options and configuration can be write as
 /// `rstest` attribute arguments. This syntax is a little less verbose but make
-/// composition harder: for istance try to add some cases to a `rstest_reuse` template
+/// composition harder: for instance try to add some cases to a `rstest_reuse` template
 /// is really hard.
 ///
 /// So we'll continue to maintain the old syntax for a long time but we strongly encourage
@@ -1177,7 +1335,7 @@ pub fn fixture(
 /// # fn is_valid(input: &str) -> bool { true }
 ///
 /// #[rstest(
-///     input => ["Jhon", "alice", "My_Name", "Zigy_2001"]
+///     input => ["John", "alice", "My_Name", "Zigy_2001"]
 /// )]
 /// fn should_be_valid(input: &str) {
 ///     assert!(is_valid(input))
@@ -1196,36 +1354,4 @@ pub fn fixture(
 /// }
 /// ```
 ///
-#[proc_macro_attribute]
-pub fn rstest(
-    args: proc_macro::TokenStream,
-    input: proc_macro::TokenStream,
-) -> proc_macro::TokenStream {
-    let mut test = parse_macro_input!(input as ItemFn);
-    let mut info = parse_macro_input!(args as RsTestInfo);
-
-    let replace_result = ReplaceFutureAttribute::replace(&mut test);
-    let extend_result = info.extend_with_function_attrs(&mut test);
-
-    let mut errors = error::rstest(&test, &info);
-
-    if let Err(attrs_errors) = replace_result {
-        attrs_errors.to_tokens(&mut errors);
-    }
-    if let Err(attrs_errors) = extend_result {
-        attrs_errors.to_tokens(&mut errors);
-    }
-
-    if errors.is_empty() {
-        if info.data.has_list_values() {
-            render::matrix(test, info)
-        } else if info.data.has_cases() {
-            render::parametrize(test, info)
-        } else {
-            render::single(test, info)
-        }
-    } else {
-        errors
-    }
-    .into()
-}
+pub use rstest_macros::rstest;
