@@ -1,14 +1,16 @@
-use std::path::Path;
-
-use super::resources;
+use std::{fs::File, io::Write, path::Path};
 
 use mytest::*;
 use rstest_test::*;
 use unindent::Unindent;
 
-fn prj(res: impl AsRef<Path>) -> Project {
+pub fn resources(res: impl AsRef<Path>) -> std::path::PathBuf {
     let path = Path::new("rstest").join(res.as_ref());
-    crate::prj().set_code_file(resources(path))
+    super::resources(path)
+}
+
+fn prj(res: impl AsRef<Path>) -> Project {
+    crate::prj().set_code_file(resources(res))
 }
 
 fn run_test(res: impl AsRef<Path>) -> (std::process::Output, String) {
@@ -19,6 +21,68 @@ fn run_test(res: impl AsRef<Path>) -> (std::process::Output, String) {
     )
 }
 
+#[test]
+fn files() {
+    let prj = prj("files.rs");
+    let files_path = prj.path().join("files");
+    let sub_folder = files_path.join("sub");
+    let up_sub_folder = prj.path().join("../files_test_sub_folder");
+    std::fs::create_dir(&files_path).unwrap();
+    std::fs::create_dir(&sub_folder).unwrap();
+    std::fs::create_dir(&up_sub_folder).unwrap();
+
+    for n in 0..4 {
+        let name = format!("element_{}.txt", n);
+        let path = files_path.join(&name);
+        let mut out = File::create(path).unwrap();
+        out.write_all(name.as_bytes()).unwrap();
+        out.write_all(b"--\n").unwrap();
+        out.write_all(b"something else\n").unwrap();
+    }
+    let dot = files_path.join(".ignore_me.txt");
+    File::create(dot)
+        .unwrap()
+        .write_all(b".ignore_me.txt--\n")
+        .unwrap();
+    let exclude = files_path.join("exclude.txt");
+    File::create(exclude)
+        .unwrap()
+        .write_all(b"excluded\n")
+        .unwrap();
+    let sub = sub_folder.join("sub_dir_file.txt");
+    File::create(sub)
+        .unwrap()
+        .write_all(b"sub_dir_file.txt--\nmore")
+        .unwrap();
+    let down_from_parent_folder = up_sub_folder.join("from_parent_folder.txt");
+    File::create(down_from_parent_folder)
+        .unwrap()
+        .write_all(b"from_parent_folder.txt--\nmore")
+        .unwrap();
+    let output = prj.run_tests().unwrap();
+
+    TestResults::new()
+        .ok("start_with_name::path_1__UP_files_test_sub_folder_from_parent_folder_txt")
+        .ok("start_with_name::path_2_files_element_0_txt")
+        .ok("start_with_name::path_3_files_element_1_txt")
+        .ok("start_with_name::path_4_files_element_2_txt")
+        .ok("start_with_name::path_5_files_element_3_txt")
+        .ok("start_with_name::path_6_files_sub_sub_dir_file_txt")
+        .ok("start_with_name_with_include::path_1_files__ignore_me_txt")
+        .ok("start_with_name_with_include::path_2_files_element_0_txt")
+        .ok("start_with_name_with_include::path_3_files_element_1_txt")
+        .ok("start_with_name_with_include::path_4_files_element_2_txt")
+        .ok("start_with_name_with_include::path_5_files_element_3_txt")
+        .ok("start_with_name_with_include::path_6_files_sub_sub_dir_file_txt")
+        .ok("module::pathbuf_need_not_be_in_scope::path_1_files__ignore_me_txt")
+        .ok("module::pathbuf_need_not_be_in_scope::path_2_files_element_0_txt")
+        .ok("module::pathbuf_need_not_be_in_scope::path_3_files_element_1_txt")
+        .ok("module::pathbuf_need_not_be_in_scope::path_4_files_element_2_txt")
+        .ok("module::pathbuf_need_not_be_in_scope::path_5_files_element_3_txt")
+        .ok("module::pathbuf_need_not_be_in_scope::path_6_files_sub_sub_dir_file_txt")
+        .assert(output);
+}
+
 #[test]
 fn mutable_input() {
     let (output, _) = run_test("mut.rs");
@@ -64,6 +128,13 @@ fn should_not_show_a_warning_for_should_panic_attribute() {
     assert!(!output.stderr.str().contains("unused attribute"));
 }
 
+#[test]
+fn should_not_show_a_warning_for_values_test_names() {
+    let (output, _) = run_test("values_tests_name.rs");
+
+    assert_not_in!(output.stderr.str(), "warning:");
+}
+
 #[test]
 fn should_map_fixture_by_remove_first_underscore_if_any() {
     let (output, _) = run_test("remove_underscore.rs");
@@ -98,10 +169,23 @@ fn use_mutable_fixture_in_parametric_argumnts() {
     let (output, _) = run_test("use_mutable_fixture_in_parametric_argumnts.rs");
 
     TestResults::new()
+        .with_contains(true)
         .ok("use_mutate_fixture::case_1::b_1")
         .assert(output);
 }
 
+#[test]
+fn should_not_remove_lifetimes() {
+    let (output, _) = run_test("lifetimes.rs");
+
+    TestResults::new()
+        .with_contains(true)
+        .ok("case")
+        .ok("values")
+        .ok("fixture")
+        .assert(output);
+}
+
 #[test]
 fn should_reject_no_item_function() {
     let (output, name) = run_test("reject_no_item_function.rs");
@@ -164,16 +248,13 @@ mod dump_input_values {
 
         TestResults::new()
             .fail("single_fail")
+            .fail("no_trace_single_fail")
             .fail("cases_fail::case_1")
             .fail("cases_fail::case_2")
-            .fail("matrix_fail::u_1::s_1::t_1")
-            .fail("matrix_fail::u_1::s_1::t_2")
-            .fail("matrix_fail::u_1::s_2::t_1")
-            .fail("matrix_fail::u_1::s_2::t_2")
-            .fail("matrix_fail::u_2::s_1::t_1")
-            .fail("matrix_fail::u_2::s_1::t_2")
-            .fail("matrix_fail::u_1::s_2::t_1")
-            .fail("matrix_fail::u_2::s_2::t_2")
+            .fail("no_trace_cases_fail::case_1")
+            .fail("no_trace_cases_fail::case_2")
+            .fail_with("matrix_fail::u_1", false, 8)
+            .fail_with("matrix_fail::u_2", false, 8)
             .assert(output);
 
         assert_in!(out, "fu32 = 42");
@@ -195,6 +276,15 @@ mod dump_input_values {
         assert_in!(out, "u = 2");
         assert_in!(out, r#"s = "srt""#);
         assert_in!(out, r#"t = ("TT", -24)"#);
+
+        let expected = 11;
+        for marker in ["TEST START", "TEST ARGUMENTS"] {
+            let n_found = out.lines().filter(|l| l.contains(marker)).count();
+            assert_eq!(
+                n_found, expected,
+                "Should contain {expected} '{marker}' but found {n_found}. [Should not enclose output if no trace]"
+            );
+        }
     }
 
     #[rstest]
@@ -235,7 +325,7 @@ mod dump_input_values {
         TestResults::new()
             .fail("simple")
             .fail("cases::case_1")
-            .fail("matrix::a_1::b_1::dd_1")
+            .fail_in("matrix::a_1")
             .assert(output);
 
         assert_in!(out, "fu32 = 42");
@@ -257,10 +347,11 @@ mod dump_input_values {
             .take_while(|l| !l.contains("TEST START"))
             .collect::<Vec<_>>();
 
+        let expected = 4;
         assert_eq!(
-            4,
+            expected,
             lines.len(),
-            "Not contains 3 lines but {}: '{}'",
+            "Not contains {expected} lines but {}: '{}'",
             lines.len(),
             lines.join("\n")
         );
@@ -309,9 +400,12 @@ mod single {
             .assert(output);
     }
 
-    #[test]
-    fn should_run_async_function() {
-        let prj = prj(res("async.rs"));
+    #[rstest]
+    #[case("async.rs")]
+    #[case("async_awt.rs")]
+    #[case("async_awt_global.rs")]
+    fn should_run_async_function(#[case] name: &str) {
+        let prj = prj(res(name));
         prj.add_dependency("async-std", r#"{version="*", features=["attributes"]}"#);
 
         let output = prj.run_tests().unwrap();
@@ -439,9 +533,12 @@ mod cases {
             .assert(output);
     }
 
-    #[test]
-    fn should_run_async_function() {
-        let prj = prj(res("async.rs"));
+    #[rstest]
+    #[case("async.rs")]
+    #[case("async_awt.rs")]
+    #[case("async_awt_global.rs")]
+    fn should_run_async_function(#[case] name: &str) {
+        let prj = prj(res(name));
         prj.add_dependency("async-std", r#"{version="*", features=["attributes"]}"#);
 
         let output = prj.run_tests().unwrap();
@@ -455,6 +552,19 @@ mod cases {
             .assert(output);
     }
 
+    #[rstest]
+    fn should_run_async_mut() {
+        let prj = prj(res("async_awt_mut.rs"));
+        prj.add_dependency("async-std", r#"{version="*", features=["attributes"]}"#);
+
+        let output = prj.run_tests().unwrap();
+
+        TestResults::new()
+            .ok("my_mut_test_global_awt::case_1_pass")
+            .ok("my_mut_test_local_awt::case_1_pass")
+            .assert(output);
+    }
+
     #[test]
     fn should_use_injected_test_attr() {
         let prj = prj(res("inject.rs"));
@@ -722,10 +832,11 @@ mod matrix {
         let (output, _) = run_test(res("simple.rs"));
 
         TestResults::new()
-            .ok("strlen_test::expected_1::input_1")
-            .ok("strlen_test::expected_1::input_2")
-            .ok("strlen_test::expected_2::input_1")
-            .ok("strlen_test::expected_2::input_2")
+            .with_contains(true)
+            .ok("strlen_test::expected_1_4::input_1___ciao__")
+            .ok("strlen_test::expected_1_4::input_2___buzz__")
+            .ok("strlen_test::expected_2_2_3_2::input_1___ciao__")
+            .ok("strlen_test::expected_2_2_3_2::input_2___buzz__")
             .assert(output);
     }
 
@@ -734,49 +845,45 @@ mod matrix {
         let (output, _) = run_test(res("partial.rs"));
 
         TestResults::new()
-            .ok("default::a_1::b_1")
-            .ok("default::a_1::b_2")
-            .ok("default::a_2::b_1")
-            .ok("partial_2::a_2::b_2")
-            .ok("partial_attr_2::a_2::b_2")
-            .ok("complete::a_2::b_2")
-            .ok("complete_attr::a_2::b_2")
-            .fail("default::a_2::b_2")
-            .fail("partial_1::a_1::b_1")
-            .fail("partial_1::a_1::b_2")
-            .fail("partial_1::a_2::b_1")
-            .fail("partial_1::a_2::b_2")
-            .fail("partial_2::a_1::b_1")
-            .fail("partial_2::a_1::b_2")
-            .fail("partial_2::a_2::b_1")
-            .fail("complete::a_1::b_1")
-            .fail("complete::a_1::b_2")
-            .fail("complete::a_2::b_1")
-            .fail("partial_attr_1::a_1::b_1")
-            .fail("partial_attr_1::a_1::b_2")
-            .fail("partial_attr_1::a_2::b_1")
-            .fail("partial_attr_1::a_2::b_2")
-            .fail("partial_attr_2::a_1::b_1")
-            .fail("partial_attr_2::a_1::b_2")
-            .fail("partial_attr_2::a_2::b_1")
-            .fail("complete_attr::a_1::b_1")
-            .fail("complete_attr::a_1::b_2")
-            .fail("complete_attr::a_2::b_1")
+            .with_contains(true)
+            .ok_times("default::a_1", 2)
+            .ok("default::a_2")
+            .ok("partial_2::a_2")
+            .ok("partial_attr_2::a_2")
+            .ok("complete::a_2")
+            .ok("complete_attr::a_2")
+            .fail("default::a_2")
+            .fail_times("partial_1::a_1", 2)
+            .fail_times("partial_1::a_2", 2)
+            .fail_times("partial_2::a_1", 2)
+            .fail("partial_2::a_2")
+            .fail_times("complete::a_1", 2)
+            .fail("complete::a_2")
+            .fail_times("partial_attr_1::a_1", 2)
+            .fail_times("partial_attr_1::a_2", 2)
+            .fail_times("partial_attr_2::a_1", 2)
+            .fail("partial_attr_2::a_2")
+            .fail_times("complete_attr::a_1", 2)
+            .fail("complete_attr::a_2")
             .assert(output);
     }
 
-    #[test]
-    fn should_run_async_function() {
-        let prj = prj(res("async.rs"));
+    #[rstest]
+    #[case("async.rs")]
+    #[case("async_awt.rs")]
+    #[case("async_awt_global.rs")]
+    fn should_run_async_function(#[case] name: &str) {
+        let prj = prj(res(name));
         prj.add_dependency("async-std", r#"{version="*", features=["attributes"]}"#);
 
         let output = prj.run_tests().unwrap();
 
         TestResults::new()
-            .ok("my_async_test::first_1::second_1")
-            .fail("my_async_test::first_1::second_2")
-            .fail("my_async_test::first_2::second_1")
-            .ok("my_async_test::first_2::second_2")
+            .with_contains(true)
+            .ok("my_async_test::first_1")
+            .fail("my_async_test::first_1")
+            .fail("my_async_test::first_2")
+            .ok("my_async_test::first_2")
             .assert(output);
     }
 
@@ -788,14 +895,15 @@ mod matrix {
         let output = prj.run_tests().unwrap();
 
         TestResults::new()
-            .ok("sync::first_1::second_1")
-            .fail("sync::first_1::second_2")
-            .fail("sync::first_2::second_1")
-            .ok("sync::first_2::second_2")
-            .ok("fn_async::first_1::second_1")
-            .fail("fn_async::first_1::second_2")
-            .fail("fn_async::first_2::second_1")
-            .ok("fn_async::first_2::second_2")
+            .with_contains(true)
+            .ok("sync::first_1")
+            .fail("sync::first_1")
+            .fail("sync::first_2")
+            .ok("sync::first_2")
+            .ok("fn_async::first_1")
+            .fail("fn_async::first_1")
+            .fail("fn_async::first_2")
+            .ok("fn_async::first_2")
             .assert(output);
     }
 
@@ -804,18 +912,18 @@ mod matrix {
         let (output, _) = run_test(res("use_attr.rs"));
 
         TestResults::new()
-            .ok("both::expected_1::input_1")
-            .ok("both::expected_1::input_2")
-            .ok("both::expected_2::input_1")
-            .ok("both::expected_2::input_2")
-            .ok("first::input_1::expected_1")
-            .ok("first::input_2::expected_1")
-            .ok("first::input_1::expected_2")
-            .ok("first::input_2::expected_2")
-            .ok("second::expected_1::input_1")
-            .ok("second::expected_1::input_2")
-            .ok("second::expected_2::input_1")
-            .ok("second::expected_2::input_2")
+            .ok("both::expected_1_4::input_1___ciao__")
+            .ok("both::expected_1_4::input_2___buzz__")
+            .ok("both::expected_2_2_3_2::input_1___ciao__")
+            .ok("both::expected_2_2_3_2::input_2___buzz__")
+            .ok("first::input_1___ciao__::expected_1_4")
+            .ok("first::input_2___buzz__::expected_1_4")
+            .ok("first::input_1___ciao__::expected_2_2_3_2")
+            .ok("first::input_2___buzz__::expected_2_2_3_2")
+            .ok("second::expected_1_4::input_1___ciao__")
+            .ok("second::expected_1_4::input_2___buzz__")
+            .ok("second::expected_2_2_3_2::input_1___ciao__")
+            .ok("second::expected_2_2_3_2::input_2___buzz__")
             .assert(output);
     }
 }
@@ -824,7 +932,10 @@ mod matrix {
 fn convert_string_literal() {
     let (output, _) = run_test("convert_string_literal.rs");
 
-    assert_in!(output.stdout.str(), "Cannot parse 'error' to get MyType");
+    assert_regex!(
+        "Cannot parse 'error' to get [a-z:_0-9]*MyType",
+        output.stdout.str()
+    );
 
     TestResults::new()
         .ok("cases::case_1")
@@ -833,11 +944,11 @@ fn convert_string_literal() {
         .ok("cases::case_4")
         .fail("cases::case_5")
         .fail("cases::case_6")
-        .ok("values::addr_1")
-        .ok("values::addr_2")
-        .fail("values::addr_3")
-        .fail("values::addr_4")
-        .ok("not_convert_byte_array::case_1::values_1")
+        .ok_in("values::addr_1")
+        .ok_in("values::addr_2")
+        .fail_in("values::addr_3")
+        .fail_in("values::addr_4")
+        .ok_in("not_convert_byte_array::case_1::values_1")
         .ok("not_convert_impl::case_1")
         .ok("not_convert_generics::case_1")
         .ok("not_convert_generics::case_2")
@@ -851,14 +962,14 @@ fn happy_path() {
     let (output, _) = run_test("happy_path.rs");
 
     TestResults::new()
-        .ok("happy::case_1::expected_1::input_1")
-        .ok("happy::case_1::expected_1::input_2")
-        .ok("happy::case_1::expected_2::input_1")
-        .ok("happy::case_1::expected_2::input_2")
-        .ok("happy::case_2_second::expected_1::input_1")
-        .ok("happy::case_2_second::expected_1::input_2")
-        .ok("happy::case_2_second::expected_2::input_1")
-        .ok("happy::case_2_second::expected_2::input_2")
+        .ok("happy::case_1::expected_1_4::input_1___ciao__")
+        .ok("happy::case_1::expected_1_4::input_2___buzz__")
+        .ok("happy::case_1::expected_2_2_3_2::input_1___ciao__")
+        .ok("happy::case_1::expected_2_2_3_2::input_2___buzz__")
+        .ok("happy::case_2_second::expected_1_4::input_1___ciao__")
+        .ok("happy::case_2_second::expected_1_4::input_2___buzz__")
+        .ok("happy::case_2_second::expected_2_2_3_2::input_1___ciao__")
+        .ok("happy::case_2_second::expected_2_2_3_2::input_2___buzz__")
         .assert(output);
 }
 
@@ -868,8 +979,10 @@ fn rename() {
 
     TestResults::new()
         .ok("compact")
+        .ok("compact_mod")
         .ok("compact_injected")
         .ok("attribute")
+        .ok("attribute_mod")
         .ok("attribute_injected")
         .assert(output);
 }
@@ -879,6 +992,7 @@ fn ignore_underscore_args() {
     let (output, _) = run_test("ignore_args.rs");
 
     TestResults::new()
+        .with_contains(true)
         .ok("test::case_1::_ignore3_1")
         .ok("test::case_1::_ignore3_2")
         .ok("test::case_1::_ignore3_3")
@@ -890,6 +1004,192 @@ fn ignore_underscore_args() {
         .assert(output);
 }
 
+#[test]
+fn timeout() {
+    let mut prj = prj("timeout.rs");
+    prj.add_dependency("async-std", r#"{version="*", features=["attributes"]}"#);
+    prj.set_default_timeout(1);
+    let output = prj.run_tests().unwrap();
+
+    TestResults::new()
+        .ok("thread::single_pass")
+        .fail("thread::single_fail_value")
+        .ok("thread::fail_with_user_message")
+        .fail("thread::single_fail_timeout")
+        .ok("thread::one_pass::case_1")
+        .fail("thread::one_fail_value::case_1")
+        .fail("thread::one_fail_timeout::case_1")
+        .ok("thread::group_same_timeout::case_1_pass")
+        .fail("thread::group_same_timeout::case_2_fail_timeout")
+        .fail("thread::group_same_timeout::case_3_fail_value")
+        .ok("thread::group_single_timeout::case_1_pass")
+        .fail("thread::group_single_timeout::case_2_fail_timeout")
+        .fail("thread::group_single_timeout::case_3_fail_value")
+        .ok("thread::group_one_timeout_override::case_1_pass")
+        .fail("thread::group_one_timeout_override::case_2_fail_timeout")
+        .fail("thread::group_one_timeout_override::case_3_fail_value")
+        .ok("thread::compile_with_no_copy_arg::case_1")
+        .ok("thread::compile_with_no_copy_fixture")
+        .fail("thread::default_timeout_failure")
+        .ok("async_std_cases::single_pass")
+        .fail("async_std_cases::single_fail_value")
+        .ok("async_std_cases::fail_with_user_message")
+        .fail("async_std_cases::single_fail_timeout")
+        .ok("async_std_cases::one_pass::case_1")
+        .fail("async_std_cases::one_fail_value::case_1")
+        .fail("async_std_cases::one_fail_timeout::case_1")
+        .ok("async_std_cases::group_same_timeout::case_1_pass")
+        .fail("async_std_cases::group_same_timeout::case_2_fail_timeout")
+        .fail("async_std_cases::group_same_timeout::case_3_fail_value")
+        .ok("async_std_cases::group_single_timeout::case_1_pass")
+        .fail("async_std_cases::group_single_timeout::case_2_fail_timeout")
+        .fail("async_std_cases::group_single_timeout::case_3_fail_value")
+        .ok("async_std_cases::group_one_timeout_override::case_1_pass")
+        .fail("async_std_cases::group_one_timeout_override::case_2_fail_timeout")
+        .fail("async_std_cases::group_one_timeout_override::case_3_fail_value")
+        .ok("async_std_cases::compile_with_no_copy_arg::case_1")
+        .ok("async_std_cases::compile_with_no_copy_fixture")
+        .ok("async_std_cases::compile_with_async_fixture")
+        .ok("async_std_cases::compile_with_async_awt_fixture")
+        .assert(output);
+}
+
+mod import_crate_with_other_name {
+    use super::*;
+
+    fn prj(res: &str, features: Option<&[&str]>) -> Project {
+        let prj = crate::base_prj();
+        let default_features = features.is_none();
+        let features = features
+            .map(|features| {
+                features
+                    .iter()
+                    .map(|f| format!(r#""{}""#, f))
+                    .collect::<Vec<_>>()
+                    .join(",")
+            })
+            .unwrap_or_else(|| "".to_string());
+        prj.add_dependency(
+            "other_name",
+            &format!(
+                r#"{{path="{}", package = "rstest", default-features = {}, features = [{}]}}"#,
+                prj.exec_dir_str().as_str(),
+                default_features,
+                features
+            ),
+        );
+        prj.set_code_file(resources(res))
+    }
+
+    #[test]
+    fn should_fails_to_compile_if_crate_name_feature_is_not_enabled() {
+        let prj = prj("timeout_other_name.rs", Some(&[]));
+        assert!(!prj.compile().unwrap().status.success());
+    }
+
+    #[test]
+    fn should_always_compile_project_that_use_default_name() {
+        let prj = crate::base_prj();
+        prj.add_dependency(
+            "rstest",
+            &format!(
+                r#"{{path="{}", default-features = false}}"#,
+                prj.exec_dir_str().as_str(),
+            ),
+        );
+        let prj = prj.set_code_file(resources("convert_string_literal.rs"));
+
+        assert!(prj.compile().unwrap().status.success());
+    }
+
+    #[rstest]
+    #[case::default_features(None)]
+    #[case::with_crate_name_feature(Some(["crate-name"].as_slice()))]
+    fn timeout_should_compile_and_run(#[case] features: Option<&[&str]>) {
+        let prj = prj("timeout_other_name.rs", features);
+        assert!(prj.compile().unwrap().status.success());
+    }
+
+    #[rstest]
+    #[case::default(None)]
+    #[case::with_crate_name_feature(Some(["crate-name"].as_slice()))]
+    fn convert_string_literal_should_compile_and_run(#[case] features: Option<&[&str]>) {
+        let prj = prj("convert_string_literal_other_name.rs", features);
+        assert!(prj.compile().unwrap().status.success());
+    }
+}
+
+#[test]
+fn local_lifetime() {
+    let (output, _) = run_test("local_lifetime.rs");
+
+    TestResults::new()
+        .ok("tests::it_works::case_1")
+        .assert(output);
+}
+
+#[test]
+fn by_ref() {
+    let prj = prj("by_ref.rs");
+    let files_path = prj.path().join("files");
+    std::fs::create_dir(&files_path).unwrap();
+    let name = "my_name.txt";
+    let mut out = File::create(files_path.join(name)).unwrap();
+    out.write_all(name.as_bytes()).unwrap();
+    let output = prj.run_tests().unwrap();
+
+    TestResults::new()
+        .ok("test::case_1::v_1_42")
+        .ok("test::case_1::v_2_142")
+        .ok("start_with_name::path_1_files_my_name_txt")
+        .assert(output);
+}
+
+mod async_timeout_feature {
+    use super::*;
+
+    fn build_prj(features: &[&str]) -> Project {
+        let prj = crate::base_prj();
+        let features = match features.is_empty() {
+            true => String::new(),
+            false => format!(r#", features=["{}"]"#, features.join(r#"",""#)),
+        };
+        prj.add_dependency(
+            "rstest",
+            &format!(
+                r#"{{path="{}", default-features = false {}}}"#,
+                prj.exec_dir_str().as_str(),
+                features
+            ),
+        );
+        prj.add_dependency("async-std", r#"{version="*", features=["attributes"]}"#);
+        prj
+    }
+
+    #[test]
+    fn should_not_compile_if_feature_disable() {
+        let prj = build_prj(&[]);
+        let output = prj
+            .set_code_file(resources("timeout_async.rs"))
+            .run_tests()
+            .unwrap();
+
+        assert_in!(output.stderr.str(), "error: Enable async-timeout feature");
+    }
+
+    #[test]
+    fn should_work_if_feature_enabled() {
+        let prj = build_prj(&["async-timeout"]);
+
+        let output = prj
+            .set_code_file(resources("timeout_async.rs"))
+            .run_tests()
+            .unwrap();
+
+        TestResults::new().ok("single_pass").assert(output);
+    }
+}
+
 mod should_show_correct_errors {
     use std::process::Output;
 
@@ -974,7 +1274,7 @@ mod should_show_correct_errors {
                   --> {}/src/lib.rs:16:29
                    |
                 16 | fn error_fixture_wrong_type(fixture: String, f: u32) {{}}
-                   |                             ^^^^^^^",
+                ",
                 name
             )
             .unindent()
@@ -1021,34 +1321,32 @@ mod should_show_correct_errors {
     fn if_arbitrary_rust_code_has_some_errors() {
         let (output, name) = execute();
 
-        assert_in!(
-            output.stderr.str(),
+        assert_regex!(
             format!(
-                "
-                error[E0308]: mismatched types
-                  --> {}/src/lib.rs:22:31
-                   |
-                22 |     case(vec![1,2,3].contains(2)))
-                   |                               ^
-                   |                               |",
+                r#"error\[E0308\]: mismatched types
+                \s+--> {}/src/lib\.rs:22:31"#,
                 name
             )
-            .unindent()
+            .unindent(),
+            output.stderr.str()
+        );
+        assert_regex!(
+            r#"22\s+|\s+case\(vec!\[1,2,3\]\.contains\(2\)\)\)"#,
+            output.stderr.str()
         );
 
-        assert_in!(
-            output.stderr.str(),
+        assert_regex!(
             format!(
-                "
-                error[E0308]: mismatched types
-                  --> {}/src/lib.rs:53:45
-                   |
-                53 | #[rstest(condition => [vec![1,2,3].contains(2)] )]
-                   |                                             ^
-                   |                                             |",
+                r#"error\[E0308\]: mismatched types
+                \s+--> {}/src/lib\.rs:53:45"#,
                 name
             )
-            .unindent()
+            .unindent(),
+            output.stderr.str()
+        );
+        assert_regex!(
+            r#"53\s+|\s+#\[rstest\(condition => \[vec!\[1,2,3\]\.contains\(2\)\] \)\]"#,
+            output.stderr.str()
         );
     }
 
@@ -1282,7 +1580,7 @@ mod should_show_correct_errors {
         assert_in!(output.stderr.str(), format!("--> {}/src/lib.rs:84:1", name));
         assert_in!(
             output.stderr.str(),
-            "| --------- doesn't satisfy `S: FromStr`"
+            "| -------- doesn't satisfy `S: FromStr`"
         );
     }
 
@@ -1294,10 +1592,10 @@ mod should_show_correct_errors {
             output.stderr.str(),
             format!(
                 "
-                  --> {}/src/lib.rs:90:57
+                  --> {}/src/lib.rs:93:8
                    |
-                90 | async fn error_future_on_impl_type(#[case] #[future] s: impl AsRef<str>) {{}}
-                   |                                                         ^^^^^^^^^^^^^^^
+                93 |     s: impl AsRef<str>,
+                   |        ^^^^^^^^^^^^^^^
                 ",
                 name
             )
@@ -1313,14 +1611,104 @@ mod should_show_correct_errors {
             output.stderr.str(),
             format!(
                 "
-                  --> {}/src/lib.rs:94:54
-                   |
-                94 | async fn error_future_on_impl_type(#[case] #[future] #[future] a: i32) {{}}
-                   |                                                      ^^^^^^^^^
+                   --> {}/src/lib.rs:102:5
+                    |
+                102 |     #[future]
+                    |     ^^^^^^^^^
                 ",
                 name
             )
             .unindent()
         );
     }
+
+    #[test]
+    fn if_use_timeout_without_arg() {
+        let (output, name) = execute();
+
+        assert_in!(
+            output.stderr.str(),
+            format!(
+                "
+                error: expected attribute arguments in parentheses: #[timeout(...)]
+                   --> {}/src/lib.rs:108:3
+                    |
+                108 | #[timeout]
+                    |   ^^^^^^^
+                ",
+                name
+            )
+            .unindent()
+        );
+    }
+
+    #[test]
+    fn if_timeout_is_not_an_expression() {
+        let (output, name) = execute();
+
+        assert_in!(
+            output.stderr.str(),
+            format!(
+                "
+                error: expected an expression
+                   --> {}/src/lib.rs:112:17
+                    |
+                112 | #[timeout(some -> strange -> invalid -> expression)]
+                    |                 ^
+                ",
+                name
+            )
+            .unindent()
+        );
+    }
+
+    #[test]
+    fn if_timeout_is_not_a_duration() {
+        let (output, name) = execute();
+
+        assert_in!(
+            output.stderr.str(),
+            format!(
+                "
+                error[E0308]: mismatched types
+                   --> {}/src/lib.rs:116:11",
+                name
+            )
+            .unindent()
+        );
+
+        assert_in!(
+            output.stderr.str(),
+            "
+            116 | #[timeout(42)]
+                |           ^^ expected `Duration`, found integer
+            "
+            .unindent()
+        );
+    }
+
+    #[test]
+    fn if_files_contains_absolute_path() {
+        let (output, name) = execute();
+
+        assert_in!(
+            output.stderr.str(),
+            format!(
+                "
+                error: Invalid glob path: path contains non-relative component
+                   --> {}/src/lib.rs:120:30",
+                name
+            )
+            .unindent()
+        );
+
+        assert_in!(
+            output.stderr.str(),
+            r#"
+                120 | fn error_absolute_path_files(#[files("/tmp/tmp.Q81idVZYAV/*.txt")] path: std::path::PathBuf) {}
+                    |                              ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+            "#
+            .unindent()
+        );
+    }
 }
